# Dell warranty update script for Miradore Management Suite v2.0
# Miradore Ltd 
# Last change: 27.04.2022 by OMe

#Requires -Version 5.1
function Write-LogFile {
	[CmdletBinding()]
	param (
		[Parameter(Mandatory=$True)]
        [string[]]$Level,
		[Parameter(Mandatory=$True)]
        [string[]]$Message
	)
	$TimeStamp = Get-Date -Format $conf.DateFormat
	$LogEntry = $TimeStamp + " " + $Level + " " + $Message

	Write-Output $LogEntry | Tee-Object -FilePath $LogFile -Append
}

Set-Location $PSScriptRoot
$PSDefaultParameterValues = @{'Out-File:Encoding' = 'UTF8'}

# Load settings from configuration file, exit if config.json is missing
if((Test-Path -Path ".\config.json") -eq $false){
	Write-Output "config.json is missing from the script directory, cannot run. Please add the file and run the script again."
	exit 88;
}
else {
	$conf = Get-Content ".\config.json" | ConvertFrom-Json
}

# Date format of log file name
$LogFileDate = Get-Date -Format $conf.LogFileNameDateFormat
# Log file name
$LogFile = $conf.LogFilePrefix + $LogFileDate + ".log"
# Remove old log files
Get-ChildItem -Include "$($conf.LogFilePrefix)*.log" -Recurse | Where-Object { $_.CreationTime -lt (Get-Date).AddDays(-$($conf.KeepLogFilesForDays)).Date } | Remove-Item -Force

# Ignore SSL errors, if needed
If($conf.MMSIgnoreSSLErrors -eq $true){
add-type @"
    using System.Net;
    using System.Security.Cryptography.X509Certificates;
    public class TrustAllCertsPolicy : ICertificatePolicy {
        public bool CheckValidationResult(
            ServicePoint srvPoint, X509Certificate certificate,
            WebRequest request, int certificateProblem) {
            return true;
        }
    }
"@

 [System.Net.ServicePointManager]::CertificatePolicy = New-Object TrustAllCertsPolicy
}

# TLS 1.2 support
[Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Tls12

# Check whether we have MMS credentials entered. If not, query user.
If((Test-Path -Path $conf.MMSCredFileName) -eq $false)
{
	$MMSCred = Get-Credential -Message "Miradore Management Suite web service API credentials"

	# Save the username and encrypted password
	# Can be decrypted by the same user account later

	$MMSCred | Export-Clixml -Path $conf.MMSCredFileName
}
Else
{
	Try
	{
		$MMSCred = Import-Clixml -Path $conf.MMSCredFileName
	}
	Catch
	{
		$ErrorMessage = $_.Exception.Message
		Write-LogFile -Level "ERROR" -Message "Setting of Miradore Management Suite credentials failed: $ErrorMessage"
		Write-LogFile -Level "ERROR" -Message "You're probably running as a different user than the one used to save the credentials. Remove $($conf.MMSCredFileName) and run once manually with $env:username to save working credentials."
		exit 1;
	}
}

# Check whether we have Dell API credentials entered. If not, query user.
If((Test-Path -Path $conf.DellAPICredFileName) -eq $false)
{
	$DellAPICred = Get-Credential -Message "Dell API credentials, type API Key as user name and Secret as password"

	# Save the username and encrypted password
	# Can be decrypted by the same user account later

	$DellAPICred | Export-Clixml -Path $conf.DellAPICredFileName
}
Else
{
	Try
	{
		$DellAPICred = Import-Clixml -Path $conf.DellAPICredFileName
	}
	Catch
	{
		$ErrorMessage = $_.Exception.Message
		Write-LogFile -Level "ERROR" -Message "Setting of Dell API credentials failed: $ErrorMessage"
		Write-LogFile -Level "ERROR" -Message "You're probably running as a different user than the one used to save the credentials. Remove $($conf.DellAPICredFileName) and run once manually with $env:username to save working credentials."
		exit 1;
	}
}

# Generate credentials for the MMS web request
$user = $MMSCred.UserName
$pass = $MMSCred.GetNetworkCredential().Password
$pair = "${user}:${pass}"
$bytes = [System.Text.Encoding]::ASCII.GetBytes($pair)
$base64 = [System.Convert]::ToBase64String($bytes)
$basicAuthValue = "Basic $base64"
$MMSheaders = @{ Authorization = $basicAuthValue }

Write-LogFile -Level "INFO" -Message "Script is starting to update warranty information for Dell assets (log only: $($conf.LogOnly))."

#
# 1. Get all non-removed Dell assets from MMS
#

$CurPageItems = $conf.MMSAPIItemsPerQuery
$CurrentPage = 1
$MMSDellAssetList = @()

While($CurPageItems -eq $conf.MMSAPIItemsPerQuery)
{
	$CurPageItems = 0
	$APIQuery = "/Connectors/Rest.svc/Asset?select=Asset[SerialNumber,WarrantyStart,WarrantyEnd]&filters=Asset.Status ne 'Removed' AND Asset.Model.Manufacturer.Name eq 'Dell' AND Asset.Model.HWCategory.Type eq 'Computer'&options=rows=$($conf.MMSAPIItemsPerQuery),page=$CurrentPage,type=embedded,df=yyyy-MM-dd"
	$WebRequest = "$($conf.MMSProtocol)://$($conf.MMSServerHostName):$($conf.MMSHTTPPort)/$($conf.MMSInstance)$($APIQuery)"
	# Send the web request and handle errors
	Try
	{
		$MMSDellAssets = [system.Text.Encoding]::UTF8.GetString((Invoke-WebRequest -uri $WebRequest -Headers $MMSheaders -ContentType "text/plain; charset=utf-8").RawContentStream.ToArray())
	}
	Catch
	{
		# Log errors, if any
		$ErrorMessage = $_.Exception.Message
		Write-LogFile -Level "ERROR" -Message "Calling the Miradore web service for the asset data failed: $ErrorMessage"

		If($ErrorMessage -like "*Forbidden*")
		{
			Write-LogFile -Level "ERROR" -Message "Erroneous username or password, or user not in the Web service readers group."
		}
			
		exit 1;
	}
	# Parse the MMS assets data into XML
	[xml]$MMSDellAssetsXml = $MMSDellAssets
    
    # Save found assets to variable for later use
    $MMSDellAssetList += $MMSDellAssetsXml.Response.Items.Asset

    # Update counter on how many assets handled this time
    $CurPageItems = $CurPageItems + $MMSDellAssetsXml.Response.Items.count

    # Increase the page number to get the next page if available
	$CurrentPage++
}

#
# 2. Generate access token for Dell API, valid for 1 hour
#

$DellOAuthURL = "https://apigtwb2c.us.dell.com/auth/oauth/v2/token"
$DellClientID = $DellAPICred.UserName
$DellClientSecret = $DellAPICred.GetNetworkCredential().Password

$TokenRequestBody = @{
    client_id = $DellClientID
    client_secret = $DellClientSecret
    grant_type = 'client_credentials'
}
 
$PostSplat = @{
    ContentType = 'application/x-www-form-urlencoded'
    Method = 'POST'
    Body = $TokenRequestBody
    Uri = $DellOAuthURL
}

try {
    $TokenRequest = Invoke-RestMethod @PostSplat
}
catch {
    $ErrorMessage = $_.Exception.Message
	Write-LogFile -Level "ERROR" -Message "Getting Dell API access token failed: $ErrorMessage"
	exit 98;
}
$DellAPIAccessToken = $TokenRequest.access_token
$DellAPIHeaders = @{"Authorization" = "Bearer "+ $DellAPIAccessToken}

#
# 3. Go through all Dell assets from MMS, check warranty from Dell API and update it to MMS asset if changes needed
#

$UpdatedAssets = 0

foreach($MMSDellAsset in $MMSDellAssetList){
	$DellAPIWarrantyURL = "https://apigtwb2c.us.dell.com/PROD/sbil/eapi/v5/asset-entitlements?servicetags=$($MMSDellAsset.SerialNumber)"
	$DellAPIResponse = $null

	try {
		$DellAPIResponse = Invoke-RestMethod -Method GET -Uri $DellAPIWarrantyURL -Headers $DellAPIHeaders
	}
	catch {
		$ErrorMessage = $_.Exception.Message
		Write-LogFile -Level "ERROR" -Message "Getting warranty data from Dell API failed for serial number $($MMSDellAsset.SerialNumber) with error: $ErrorMessage"
	}

	if($DellAPIResponse.entitlements.count -gt 0) {
		$DellWarrantyStart = ($DellAPIResponse.entitlements.startDate | Measure-Object -Minimum).Minimum
		$DellWarrantyEnd = ($DellAPIResponse.entitlements.endDate | Measure-Object -Maximum).Maximum

		if($null -ne $DellWarrantyStart -and $null -ne $DellWarrantyEnd) {
			$DellWarrantyStart = $DellWarrantyStart.Substring(0,10)
			$DellWarrantyEnd = $DellWarrantyEnd.Substring(0,10)

			if($MMSDellAsset.WarrantyStart -ne $DellWarrantyStart -or $MMSDellAsset.WarrantyEnd -ne $DellWarrantyEnd) {
				if($conf.LogOnly -eq $true) {
					Write-LogFile -Level "INFO" -Message "Logging only enabled, won't do changes. Warranty should be updated for serial number $($MMSDellAsset.SerialNumber), StartDate: $($DellWarrantyStart) & EndDate: $($DellWarrantyEnd)."
				}
				else {
					$APIQuery = "/Connectors/Rest.svc/Asset/$($MMSDellAsset.id)?options=df=yyyy-MM-dd"
					$WebRequest = "$($conf.MMSProtocol)://$($conf.MMSServerHostName):$($conf.MMSHTTPPort)/$($conf.MMSInstance)$($APIQuery)"

					$RequestBody = "
					<Request>
						<Items>
							<Asset>            
								<WarrantyStart>$($DellWarrantyStart)</WarrantyStart>            
								<WarrantyEnd>$($DellWarrantyEnd)</WarrantyEnd>                    
							</Asset>                   
						</Items>
					</Request>"

					# Send the request to MMS API and handle errors
					$UpdateResult = $null
					try
					{
						$UpdateResult = Invoke-WebRequest -uri $WebRequest -Headers $MMSheaders -Method PUT -Body $RequestBody -ContentType "text/plain; charset=utf-8"
					}
					catch
					{
						$ErrorMessage = $_.Exception.Message

						$result = $_.Exception.Response.GetResponseStream()
						$reader = New-Object System.IO.StreamReader($result)
						$reader.BaseStream.Position = 0
						$reader.DiscardBufferedData()
						$responseBody = $reader.ReadToEnd();

						Write-LogFile -Level "ERROR" -Message "Failed to update asset with serial number $($MMSDellAsset.SerialNumber), error: $ErrorMessage, Response body: $responseBody"

						if($ErrorMessage -like "*Forbidden*")
						{
							Write-LogFile -Level "ERROR" -Message "Erroneous username or password, or user not in the Web service writers group."
						}
						if($ErrorMessage -like "*404*")
						{
							Write-LogFile -Level "ERROR" -Message "Please check that the HTTP method POST is allowed in IIS and firewalls for the API path."
						}
						if(($ErrorMessage -notlike "*400*") -and ($ErrorMessage -notlike "*500*"))
						{
							# Terminate on an error only if error code not 400 or 500
							exit 1;
						}
					}
					if($UpdateResult.StatusCode -eq 200)
					{
						Write-LogFile -Level "INFO" -Message "Warranty successfully updated for serial number $($MMSDellAsset.SerialNumber), StartDate: $($DellWarrantyStart) & EndDate: $($DellWarrantyEnd)."
						$UpdatedAssets++
					}
				}
			}
			else {
				Write-LogFile -Level "INFO" -Message "Warranty already correct for serial number $($MMSDellAsset.SerialNumber)."
			}
		}
		else {
			Write-LogFile -Level "ERROR" -Message "Dell API response for serial number $($MMSDellAsset.SerialNumber) included NULL warranty dates."
		}
	}
}

Write-LogFile -Level "INFO" -Message "Script has finished updating warranty information for Dell assets, $($UpdatedAssets) updates done (log only: $($conf.LogOnly))."
